@@ -15,9 +15,11 @@ import pandas as pd
 
 # allows for CL script usage if gridwxcomp not installed
 try:
-    from .spatial import get_subgrid_bounds, gridmet_zonal_stats, calc_pt_error
+    from .spatial import get_subgrid_bounds, zonal_stats, calc_pt_error
+    from .plot import station_bar_plot
 except:
-    from spatial import get_subgrid_bounds, gridmet_zonal_stats, calc_pt_error
+    from spatial import get_subgrid_bounds, zonal_stats, calc_pt_error
+    from plot import station_bar_plot
 
 class InterpGdal(object):
     """
@@ -40,6 +42,9 @@ class InterpGdal(object):
             interpolation algorithm, slightly modified from gdal defaults.
             Keys are interpolation method names, keys are dictionaries with
             parameter names keys and corresponding values.
+        var_residual_names (dict): dictionary that maps names of bias ratios 
+            to the name of their intepolated residual if they are too long for
+            storing as a field name in an ESRI shapefile (i.e. > 10 chars).
         summary_csv_path (:obj:`pathlib.Path`): absolute path object to 
             input ``summary_csv_path``.
         layers (list): list of layers in ``summary_csv_path`` to interpolate
@@ -54,7 +59,7 @@ class InterpGdal(object):
         params (dict or None): default None. After :meth:`InterpGdal.gdal_grid`
             ``params`` is updated with the last used interpolation parameters
             in the form of a dictionary with parameter names as keys.
-            
+
     Example:
         The :class:`InterpGdal` class is useful for experimenting with multiple 
         interpolation algorithms provided by gdal which are optimized and
@@ -92,7 +97,7 @@ class InterpGdal(object):
         
     """
     # constant gridMET cell size in degrees
-    CELL_SIZE = 0.041666666666666664
+    GRIDMET_RES = 0.041666666666666664
     # gdal_grid interpolation methods
     interp_methods = ('average',
                      'invdist',
@@ -155,6 +160,15 @@ class InterpGdal(object):
             'nodata': -999
         }
     }
+
+    # maps bias ratio variables names to residual names for point shapefile
+    # which must be shorter (< 11 characters) to be stored as fields
+    var_residual_names = {
+        'annual_mean': 'annual_res',
+        'growseason_mean': 'grow_res',
+        'summer_mean': 'summer_res'
+    }
+
     def __init__(self, summary_csv_path):
         
         if not Path(summary_csv_path).is_file():
@@ -179,8 +193,6 @@ class InterpGdal(object):
         """
         if not Path(out_dir).is_dir():
             os.makedirs(out_dir)
-        # old method
-        #summary_file = Path(self.summary_csv_path).name
         
         point_data = Path(self.summary_csv_path).name.replace(
                 '.csv', '_tmp.csv')
@@ -257,7 +269,8 @@ class InterpGdal(object):
 
     def gdal_grid(self, layer='all', out_dir='', interp_meth='invdist', 
                   params=None, bounds=None, nx_cells=None, ny_cells=None, 
-                  scale_factor=0.1, zonal_stats=True, options=None):
+                  scale_factor=0.1, z_stats=True, res_plot=True, grid_res=None,
+                  grid_id_name='GRIDMET_ID', options=None):
         """
         Run gdal_grid command line tool to interpolate point ratios.
         
@@ -288,10 +301,12 @@ class InterpGdal(object):
                 original gridMET fishnet to create resampling resolution. If 
                 scale_factor = 0.1, the resolution will be one tenth gridMET 
                 resolution or 400 m.
-            zonal_stats (bool): default True. Calculate zonal means of 
+            z_stats (bool): default True. Calculate zonal means of 
                 interpolated surface to gridMET cells in fishnet and save to a 
                 CSV file. The CSV file will be saved to the same directory as 
                 the interpolated raster file(s).
+            res_plot (bool): default True. Make bar plot for residual (error)
+                between interpolated and station value for ``layer``.
             options (str or None): default None. Extra command line options for
                 gdal_grid spatial interpolation.
                 
@@ -318,18 +333,29 @@ class InterpGdal(object):
             >>> # run inverse distance interpolation
             >>> test.gdal_grid(out_dir=out_dir, layer=layers)
             
-            Note, zonal statistics on gridMET cells in a fishnet grid are 
-            calculated by default, to avoid an error the fishnet must have been
-            previously created using :func:`gridwxcomp.spatial.make_grid`. After            running the code above the following files will be created in the 
-            'default_params' directory which will be build in the same location
-            as the input summary CSV::
+            Note, zonal statistics to gridMET cells and interpolated residual 
+            plots are computed by default. A gridMET fishnet must have been
+            previously created using :func:`gridwxcomp.spatial.make_grid`. 
             
-                default_params/
+            After running the code above the following files will be created 
+            in the 'default_params' directory which will be build in the same 
+            location as the input summary CSV::
+
+                default_params/           
                 ├── annual_mean.tiff
                 ├── annual_mean.vrt
+                ├── etr_mm_summary_comp_all_yrs.csv
+                ├── etr_mm_summary_pts.cpg
+                ├── etr_mm_summary_pts.dbf
+                ├── etr_mm_summary_pts.prj
+                ├── etr_mm_summary_pts.shp
+                ├── etr_mm_summary_pts.shx
+                ├── zonal_stats.csv
                 ├── growseason_mean.tiff
                 ├── growseason_mean.vrt
-                └── gridMET_stats.csv
+                └── residual_plots/
+                    ├── annual_res.html
+                    └── grow_res.html
 
             GeoTiff interpolated raster files are now created for select layers
             as well as VRT (virtual raster) meta files that store info
@@ -345,9 +371,9 @@ class InterpGdal(object):
                 508975     0.9667075970344162 0.9117676407214926 
                 ========== ================== ================== 
                 
-            On a final note, there are several :class:`InterpGdal` instance
-            attributes that may be useful, for example to see the parameters
-            that were used for the last call to :meth:`InterpGdal.gdal_grid`
+            There are several :class:`InterpGdal` instance attributes that 
+            may be useful, for example to see the parameters that were used 
+            for the last call to :meth:`InterpGdal.gdal_grid`
             
             >>> test.params
             {'power': 4,
@@ -391,16 +417,31 @@ class InterpGdal(object):
         
         cwd = os.getcwd()
         
-        out_dir = Path(self.summary_csv_path).parent / Path(out_dir).resolve()
+        # user provided uniform grid, cell size should be in dec. degrees
+        if grid_res is not None:
+            CS = grid_res
+        else: # assume gridMET
+            CS = self.GRIDMET_RES
+        self.grid_res = CS
+        
+        # grid resolution in decimal degrees
+        res = scale_factor * CS
+        # file structure to match gridwxcomp.spatial.interpolate
+        grid_var = str(self.summary_csv_path).split('_summary')[0]
+        
+        # out_dir as subdir from dir with summary CSV 
+        out_dir = (Path(self.summary_csv_path).parent/Path(out_dir)).resolve()
+
         if not out_dir.is_dir():
+            print('{}\nDoes not exist, creating directory'.format(str(out_dir)))
             out_dir.mkdir(parents=True, exist_ok=True)
     
         source_file = Path(self.summary_csv_path).name
         source = source_file.replace('.csv', '_tmp')
         
         if interp_meth not in InterpGdal.interp_methods:
-            raise KeyError('{} not a valid interpolation method'.format(
-                interp_meth))
+            err_msg = '{} not a valid interpolation method'.format(interp_meth)
+            raise KeyError(err_msg)
         self.interp_meth = interp_meth
             
         # look up default parameters for interpolation method
@@ -415,26 +456,31 @@ class InterpGdal(object):
         # update instance parameters for later reference
         self.params = params
         # parameters to command line input str :name=value:name=value ...
-        param_str = ':'+':'.join('{!s}={!r}'.format(key,val) 
+        param_str = ':'+':'.join('{}={}'.format(key,val) 
                                  for (key,val) in params.items()) 
         # get boundary info and update instance attribute
         if not bounds and not self.grid_bounds:
             # use gridwxcomp.spatial function, assume 25 cell buffer
             self.grid_bounds = get_subgrid_bounds(self.summary_csv_path, 
-                    buffer=25)
+                buffer=25, grid_res=CS)
         elif bounds:
             self.grid_bounds = bounds
         # raster extent
         xmin,xmax,ymin,ymax = self.grid_bounds
-        
-        # if not given get pixels in lon lat using gridMET resolution
+
+        # if not given get pixels in lon lat 
         # add one cell to avoid unfilled extent in case of large upscaling
         if not nx_cells:
             nx_cells = int(np.abs(xmin - xmax) / \
-                    (InterpGdal.CELL_SIZE * scale_factor)) + 1
+                    (CS * scale_factor)) + 1
         if not ny_cells:
             ny_cells = int(np.abs(ymin - ymax) / \
-                    (InterpGdal.CELL_SIZE * scale_factor)) + 1
+                    (CS * scale_factor)) + 1
+        # fix any minor adjustments to make raster fit gridMET fishnet extent
+        # if scale_factor=1 make sure raster pixels align exactly w/gridcells
+        xmax = xmin + nx_cells * CS * scale_factor
+        ymin = ymax - ny_cells * CS * scale_factor
+
         # to parse options, like --config GDAL_NUM_THREADS update here
         if not options:
             options = ''
@@ -443,7 +489,7 @@ class InterpGdal(object):
             """reuse if running multiple layers"""
             existing_layers = pd.read_csv(self.summary_csv_path).columns
             if not layer in existing_layers:
-                print('column {} does not exist in input CSV:\n {}'.format(
+                print('\nError: {} does not exist in input CSV:\n {}'.format(
                    layer, self.summary_csv_path),
                      '\nSkipping interpolation.'
                )
@@ -460,14 +506,10 @@ class InterpGdal(object):
             out_file = out_dir.joinpath(tiff_file)
             # print message to console/logging about interpolation
             grid_var = Path(self.summary_csv_path).name.split('_summ')[0]
-            # recalculate raster resolution from bounds
-            n4km_xcells = int(round(np.abs(xmin - xmax) / InterpGdal.CELL_SIZE))
-            scale_factor = n4km_xcells / nx_cells
-            res = round(4 * scale_factor * 1000)
-            _interp_msg(grid_var, layer, self.interp_meth, res, out_file) 
+            _interp_msg(grid_var, layer, self.interp_meth, res, out_file)
             # build command line arguments
             cmd = (r'gdal_grid -a {meth}{p} -txe {xmin} {xmax} -tye {ymax}' 
-                  ' {ymin} -outsize {nx} {ny} -of GTiff -ot Float64 -l {source}'
+                  ' {ymin} -outsize {nx} {ny} -of GTiff -ot Float32 -l {source}'
                   ' {vrt} {out} {options}'.format(meth=interp_meth, p=param_str,
                       xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax, nx=nx_cells, 
                       ny=ny_cells, source=source, vrt=vrt_file, out=tiff_file, 
@@ -488,9 +530,38 @@ class InterpGdal(object):
             os.chdir(cwd)
 
             # calculate interpolated values and error at stations
-            calc_pt_error(self.summary_csv_path, out_dir, layer, grid_var)
-            if zonal_stats:
-                gridmet_zonal_stats(self.summary_csv_path, out_file)
+            # only calc residuals for mean bias ratios, i.e. not std dev, etc.
+            if layer in InterpGdal.default_layers:
+                calc_pt_error(self.summary_csv_path, out_dir, layer, grid_var,
+                    grid_id_name=grid_id_name
+                )
+            else:
+                # delete tmp summary csv used in interpgdal _make_pt_vrt method 
+                # normally deleted in calc_pt_error
+                tmp_csv = str(self.summary_csv_path).replace('.csv','_tmp.csv')
+                if Path(tmp_csv).resolve().is_file():
+                    Path(tmp_csv).resolve().unlink()
+
+            # zonal means extracted to GRIDMET_ID (cell index) 
+            if z_stats:
+                zonal_stats(self.summary_csv_path, out_file,
+                    grid_id_name=grid_id_name
+                )
+            # residual (error) bar plot, only for mean bias ratios
+            if res_plot and layer in InterpGdal.default_layers:
+                layer = self.var_residual_names.get(
+                    layer, 
+                    layer.replace('mean','res')
+                )
+                y_label = 'residual (interpolated minus station value)'
+                title = 'layer: {} algorithm: {} (gdal_grid) res: {} deg.'.\
+                        format(layer, self.interp_meth, res)
+                res_plot_dir = Path(out_dir)/'residual_plots'
+                subtitle = 'parameters: {}'.format(params)
+                source_file =  Path(out_dir)/Path(self.summary_csv_path).name
+
+                station_bar_plot(source_file, layer, out_dir=res_plot_dir,
+                    y_label=y_label, title=title, subtitle=subtitle)
 
             
         # run interpolation and zonal statistics depending on layer kwarg
@@ -519,7 +590,7 @@ def _interp_msg(grid_var, layer, function, res, out_file):
         '\nInterpolating {g} point bias ratios for: {t}\n'.\
             format(g=grid_var, t=layer),
         'Using the "{}" method\n'.format(function),
-        'Resolution (pixel size) of output raster: {} m'.format(res)
+        'Resolution (pixel size) of output raster: {} degrees'.format(res)
     )
     print(
         'GeoTIFF raster will be saved to: \n',
