@@ -40,7 +40,7 @@ def daily_comparison(input_csv, out_dir=None, year_filter=None):
     Arguments:
         input_csv (str): path to input CSV file containing paired station/
             gridMET metadata. This file is created by running 
-            :mod:`gridwxcomp.prep_input` followed by :mod:`gridwxcomp.download_gridmet_ee`.
+            :mod:`gridwxcomp.prep_input` followed by :mod:`gridwxcomp.download_gridmet_opendap`.
 
     Keyword Arguments:
         out_dir (str or None): default None. Directory to save comparison 
@@ -99,7 +99,7 @@ def daily_comparison(input_csv, out_dir=None, year_filter=None):
     # List of variables to compare (STATION/gridMET ORDER SHOULD MATCH)
     station_vars = ['TMin (C)', 'TMax (C)', 'wx_Ko_c', 'Rs (w/m2)',
                     'ws_2m (m/s)', 'Vapor Pres (kPa)', 'RHAvg (%)',
-                    'Precip (mm)', 'Calc_ETo (mm)', 'Calc_ETr (mm)']
+                    'Precip (mm)', 'ETo (mm)', 'ETr (mm)']
 
     gridmet_vars = ['tmin_c', 'tmax_c', 'grid_Ko_c', 'srad_wm2', 'u2_ms',
                     'ea_kpa', 'rh_avg', 'prcp_mm', 'eto_mm', 'etr_mm']
@@ -134,18 +134,18 @@ def daily_comparison(input_csv, out_dir=None, year_filter=None):
             continue
         else:
             station_data = pd.read_excel(station_path,
-                                         sheet_name='Corrected Data')
+                sheet_name='Corrected Data', parse_dates=True, index_col=0)
             # Filter years
             if year:
                 station_data, year_str = parse_yr_filter(
                     station_data, year, label=row.STATION_ID)
             else:
-                start_yr = station_data.year.min().astype(int) 
-                end_yr = station_data.year.max().astype(int) 
+                start_yr = int(station_data.year.min()) 
+                end_yr = int(station_data.year.max()) 
                 year_str = '{}_{}'.format(start_yr, end_yr)
 
         # Import GRIDMET Data
-        grid_path = row.GRIDMET_FILE_PATH
+        grid_path = row.GRID_FILE_PATH
         # Skip if GRIDMET FILE DOES NOT EXIST
         if not os.path.exists(grid_path):
             print('SKIPPING {}. NO GRIDMET FILE FOUND.'.format(grid_path))
@@ -179,11 +179,10 @@ def daily_comparison(input_csv, out_dir=None, year_filter=None):
             grid_data['rh_avg'] = (grid_data.ea_kpa / grid_data.e_sat_kpa) * 100
 
             # Combine station and gridMET dataframes (only plotting variables)
-            merged = pd.concat([station_data[station_vars],
-                                grid_data[gridmet_vars]], axis=1,
-                               join_axes=[station_data.index])
-            # Remove results with na
-            # merged = merged.dropna()
+            #return station_data, station_vars, grid_data, gridmet_vars
+            merged = pd.concat([
+                station_data[station_vars], grid_data[gridmet_vars]], axis=1
+            )
 
             for month in range(1,13):
                 logging.info('Month: {}'.format(month))
@@ -210,7 +209,7 @@ def daily_comparison(input_csv, out_dir=None, year_filter=None):
 
                 station_vars = ['TMin (C)', 'TMax (C)', 'wx_Ko_c', 'Rs (w/m2)',
                                 'ws_2m (m/s)', 'Vapor Pres (kPa)', 'RHAvg (%)',
-                                'Precip (mm)', 'Calc_ETo (mm)', 'Calc_ETr (mm)']
+                                'Precip (mm)', 'ETo (mm)', 'ETr (mm)']
 
                 gridmet_vars = ['tmin_c', 'tmax_c', 'grid_Ko_c', 'srad_wm2',
                                 'u2_ms',
@@ -261,6 +260,9 @@ def daily_comparison(input_csv, out_dir=None, year_filter=None):
                     monthly_data2 = monthly_data[[x_var, y_var]]
                     monthly_data2 = monthly_data2.dropna()
 
+                    monthly_data2['date'] = monthly_data2.index
+                    monthly_data2.reset_index(inplace=True)
+
                     if monthly_data2.empty:
                         logging.info("Skipping {}. No Data.".format(x_var))
                         continue
@@ -268,26 +270,42 @@ def daily_comparison(input_csv, out_dir=None, year_filter=None):
                     if i == 0:
                         # Initial timeseries plot to establish xrange for link axes
                         p1 = figure(plot_width=800, plot_height=400,
-                                    x_axis_type="datetime",title = title,
+                                    title = title, x_axis_type="datetime",
                                     y_axis_label = ts_ylabel)
-                        p1.line(monthly_data2.index.to_pydatetime(),
+                        p1.line(monthly_data2.index,
                                 monthly_data2[x_var],  color="navy",
-                                alpha=0.5, legend=legendx,line_width=2)
-                        p1.line(monthly_data2.index.to_pydatetime(),
+                                alpha=0.5, legend_label=legendx,line_width=2)
+                        p1.line(monthly_data2.index,
                                 monthly_data2[y_var],  color="red",
-                                alpha=0.5, legend=legendy,line_width=2)
+                                alpha=0.5, legend_label=legendy,line_width=2)
+                        p1.xaxis.major_label_overrides = {
+                            i: date.strftime(
+                                '%Y %b %d'
+                            ) for i, date in enumerate(pd.to_datetime(
+                                monthly_data2.date
+                            )
+                        )}
+
+
                     else:
                         # Timeseries plots after first pass
                         p1 = figure(plot_width=800, plot_height=400,
-                                    x_axis_type="datetime",title = title,
-                                    y_axis_label = ts_ylabel,
+                                    title = title, x_axis_type="datetime",
+                                    y_axis_label=ts_ylabel,
                                     x_range=p1.x_range)
-                        p1.line(monthly_data2.index.to_pydatetime(),
+                        p1.line(monthly_data2.index,
                                 monthly_data2[x_var],  color="navy", alpha=0.5,
-                                legend=legendx,line_width=2)
-                        p1.line(monthly_data2.index.to_pydatetime(),
+                                legend_label=legendx,line_width=2)
+                        p1.line(monthly_data2.index,
                                 monthly_data2[y_var],  color="red", alpha=0.5,
-                                legend=legendy,line_width=2)
+                                legend_label=legendy,line_width=2)
+
+                    p1.xaxis.major_label_overrides = {
+                        i: date.strftime('%Y %b %d') for i, date in enumerate(
+                            pd.to_datetime(monthly_data2.date)
+                        )
+                    }
+
 
                     # 1 to 1 Plot
                     # Regression through Zero
@@ -313,13 +331,15 @@ def daily_comparison(input_csv, out_dir=None, year_filter=None):
                              [int(np.min([monthly_data2[y_var],
                                           monthly_data2[x_var]])-2),int(np.max(
                                  [monthly_data2[y_var],monthly_data2[x_var]])+2)],
-                              color = "black", legend = '1 to 1 line')
-                    p2.line(r_x, r_y, color="red", legend = 'Reg thru zero')
+                              color = "black", legend_label = '1 to 1 line')
+                    p2.line(r_x, r_y, color="red", legend_label = 'Reg thru zero')
                     p2.legend.location = "top_left"
 
                     # Append [p1, p2] to figure_list (create list of lists)
                     figure_list.append([p1, p2])
 
+                #return figure_list, monthly_data2
+
                 # Plot all figures in list
                 fig = gridplot(figure_list, toolbar_location="left")
 
@@ -340,7 +360,7 @@ def monthly_comparison(input_csv, out_dir=None):
         input_csv (str): path to input CSV file containing
             paired station/gridMET metadata. This file is
             created by running :mod:`gridwxcomp.prep_input` followed by
-            :mod:`gridwxcomp.download_gridmet_ee`.
+            :mod:`gridwxcomp.download_gridmet_opendap`.
 
     Keyword Arguments:
         out_dir (str): default None. Directory to save comparison plots.
@@ -391,7 +411,7 @@ def monthly_comparison(input_csv, out_dir=None):
     # List of variables to compare (STATION/gridMET ORDER SHOULD MATCH)
     station_vars = ['TMin (C)', 'TMax (C)', 'wx_Ko_c', 'Rs (w/m2)',
                     'ws_2m (m/s)', 'Vapor Pres (kPa)', 'RHAvg (%)',
-                    'Precip (mm)', 'Calc_ETo (mm)', 'Calc_ETr (mm)']
+                    'Precip (mm)', 'ETo (mm)', 'ETr (mm)']
 
     gridmet_vars = ['tmin_c', 'tmax_c', 'grid_Ko_c', 'srad_wm2', 'u2_ms',
                     'ea_kpa', 'rh_avg', 'prcp_mm', 'eto_mm', 'etr_mm']
@@ -426,11 +446,11 @@ def monthly_comparison(input_csv, out_dir=None):
                 station_path))
             continue
         else:
-            station_data = pd.read_excel(station_path,
-                                         sheet_name='Corrected Data')
+            station_data = pd.read_excel(station_path, index_col=0,
+                    parse_dates=True, sheet_name='Corrected Data')
 
         # Import GRIDMET Data
-        grid_path = row.GRIDMET_FILE_PATH
+        grid_path = row.GRID_FILE_PATH
         # Skip if GRIDMET FILE DOES NOT EXIST
         if not os.path.exists(grid_path):
             print('SKIPPING {}. NO GRIDMET FILE FOUND.'.format(grid_path))
@@ -464,12 +484,8 @@ def monthly_comparison(input_csv, out_dir=None):
 
             # Combine station and gridMET dataframes (only plotting variables)
             merged = pd.concat([station_data[station_vars],
-                                grid_data[gridmet_vars]], axis=1,
-                               join_axes=[station_data.index])
-
-            # Remove results with na
-            # merged = merged.dropna()
-
+                                grid_data[gridmet_vars]], axis=1
+            )
 
             # Monthly averages including count
             monthly = merged.groupby([lambda x: x.year, lambda x: x.month]).agg(
@@ -477,7 +493,7 @@ def monthly_comparison(input_csv, out_dir=None):
 
             # Remove months with Less Than XX Days in average
             day_limit = 10
-            monthly = monthly[monthly['Calc_ETr (mm)', 'count'] >= day_limit]
+            monthly = monthly[monthly['ETr (mm)', 'count'] >= day_limit]
 
             # Rebuild Index DateTime
             monthly['year'] = monthly.index.get_level_values(0).values
@@ -507,7 +523,7 @@ def monthly_comparison(input_csv, out_dir=None):
             station_vars = ['TMin (C)', 'TMax (C)', 'wx_Ko_c', 'Rs (w/m2)',
                             'ws_2m (m/s)', 'Vapor Pres (kPa)', 'RHAvg (%)',
                                                                'Precip (mm)',
-                            'Calc_ETo (mm)', 'Calc_ETr (mm)']
+                            'ETo (mm)', 'ETr (mm)']
 
             gridmet_vars = ['tmin_c', 'tmax_c', 'grid_Ko_c', 'srad_wm2',
                             'u2_ms',
@@ -578,10 +594,10 @@ def monthly_comparison(input_csv, out_dir=None):
                                 y_axis_label = ts_ylabel)
                     p1.line(monthly2.index.to_pydatetime(),
                             monthly2[x_var, stat],  color="navy",
-                            alpha=0.5, legend=legendx,line_width=2)
+                            alpha=0.5, legend_label=legendx,line_width=2)
                     p1.line(monthly2.index.to_pydatetime(),
                             monthly2[y_var, stat],  color="red",
-                            alpha=0.5, legend=legendy,line_width=2)
+                            alpha=0.5, legend_label=legendy,line_width=2)
                 else:
                     # Timeseries plots after first pass
                     p1 = figure(plot_width=800, plot_height=400,
@@ -590,10 +606,10 @@ def monthly_comparison(input_csv, out_dir=None):
                                 x_range=p1.x_range)
                     p1.line(monthly2.index.to_pydatetime(),
                             monthly2[x_var, stat],  color="navy", alpha=0.5,
-                            legend=legendx,line_width=2)
+                            legend_label=legendx,line_width=2)
                     p1.line(monthly2.index.to_pydatetime(),
                             monthly2[y_var, stat],  color="red", alpha=0.5,
-                            legend=legendy,line_width=2)
+                            legend_label=legendy,line_width=2)
 
                 # 1 to 1 Plot
                 # Regression through Zero
@@ -619,8 +635,8 @@ def monthly_comparison(input_csv, out_dir=None):
                          [int(np.min([monthly2[y_var, stat],
                                       monthly2[x_var, stat]])-2),int(np.max(
                              [monthly2[y_var, stat],monthly2[x_var, stat]])+2)],
-                          color = "black", legend = '1 to 1 line')
-                p2.line(r_x, r_y, color="red", legend = 'Reg thru zero')
+                          color = "black", legend_label = '1 to 1 line')
+                p2.line(r_x, r_y, color="red", legend_label = 'Reg thru zero')
                 p2.legend.location = "top_left"
 
                 # Append [p1, p2] to figure_list (create list of lists)
@@ -688,16 +704,17 @@ def station_bar_plot(summary_csv, layer, out_dir=None, x_label=None,
     """
     if not Path(summary_csv).is_file():
         err_msg = '\n{} is not a valid path to a summary CSV file!'.\
-                format(in_path)
+                format(summary_csv)
         raise FileNotFoundError(err_msg)
         
-    df = pd.read_csv(summary_csv)
+    df = pd.read_csv(summary_csv, na_values=[-999])
     
     if not layer in df.columns:
         err_msg = '\nColumn {} was not found in {}'.format(layer, summary_csv)
         raise KeyError(err_msg)
     
     df.sort_values(layer, inplace=True)
+    df.index.name = 'dummy_name' # fix internal to bokeh- reset_index 
     source = ColumnDataSource(df)
     # hover tooltip with station and value
     tooltips = [
@@ -711,10 +728,13 @@ def station_bar_plot(summary_csv, layer, out_dir=None, x_label=None,
     # save to working directory in 'station_bar_plots' if not specified
     if not out_dir:
         out_dir = Path(summary_csv).parent/'station_bar_plots'
-        if not out_dir.is_dir():
-            print('\n{} does not exist, making directory'.format(out_dir))
-            out_dir.mkdir(parents=True, exist_ok=True)
-    
+    else:
+        out_dir = Path(out_dir)
+    if not out_dir.is_dir():
+        print('\n{}\nDoes not exist, making directory'.format(
+            out_dir.absolute()))
+        out_dir.mkdir(parents=True, exist_ok=True)
+
     out_file = out_dir/'{}.html'.format(layer)
     print('\nCreating station bar plot for variable: ', layer,
          '\nUsing data from file: ', Path(summary_csv).absolute())
@@ -728,8 +748,8 @@ def station_bar_plot(summary_csv, layer, out_dir=None, x_label=None,
     
     if year_subtitle:
         # add data range (years start to end) as subtitle 
-        min_yr = df.start_year.min().astype(int)
-        max_yr = df.end_year.max().astype(int)
+        min_yr = int(df.start_year.min())
+        max_yr = int(df.end_year.max())
         if min_yr == max_yr:
             year_str = 'year: {}'.format(min_yr)
         else:
@@ -768,7 +788,7 @@ def arg_parse():
     required.add_argument(
         '-i', '--input', metavar=None, required=True,
         help='Input CSV, if type="station-grid-comp" use file created by '+\
-        'download_gridmet_ee.py, if type="station-bar" use file created '+\
+        'download_gridmet_opendap.py, if type="station-bar" use file created '+\
         'by calc_bias_ratios.py or spatial.py')
     optional.add_argument(
         '-t', '--plot-type', required=False, type=str, 
